% degeneracy vs. no degeneracy (LASSO)
% Least_SRMTL1 and Least_SRMTL2 are originally from Dr.Jieping Ye's package

clear all
load W2; % predefined W coefficient matrix
load Lm1; % degeneracy among predictors expressed through Laplacian matrix

dimt=50; % number of variables
dg=10; % degenercy group
task_num=1;
ssize=50;

for rp=1:50 % repeat 50 times
    %% generate data
    Xt1=cell(task_num,1);
    Yt1=cell(task_num,1);
    
    for kl=1:task_num
        % generate X according to network constrained
        for i=1:ssize
            for j=1:dg
                tf=normrnd(0,1);
                for e1=1:5
                    X(i,(j-1)*5+e1)=normrnd(0.9*tf,0.1);
                end
            end
        end
      
        Xt1{kl,1}=X;
        Xt1_std{kl,1}=zscore(Xt1{kl,1});
        nvar=0.1;
        e=normrnd(0,nvar,[ssize 1]);
        Yt1{kl,1}=Xt1{kl,1}*W2(:,kl)+e;
        snr(1,rp)=mean(Yt1{kl})/sqrt(nvar); % calculate signal-to-noise ratio
        Yt1_std{kl,1}=zscore(Yt1{kl,1});
    end
    %% regularization with laplacian matrix
    X_std=Xt1_std;
    Y_std=Yt1_std;
    R=eye(task_num); 
    m=1; % to count number of combination of two lambdas for transfer learning
    ls=1; % to count number of one sparse lambda for lasso
    for p2=0.1:0.1:5 % sparsity para
        %% with degeneracy
        for p1=1:1:30 % regularization parameter
            opts=[];
            [W_est funcVal] = Least_SRMTL2(X_std, Y_std, R,Lm1, p1, p2,opts); %
            % TPR and FPR for transfer learning
            tp(m,:)=zeros(1,task_num);
            fp(m,:)=zeros(1,task_num);
            for i=1:task_num
                for j=1:dimt
                    if W2(j,i)~=0 && W_est(j,i)~=0
                        tp(m,i)=tp(m,i)+1;
                    end
                    if W2(j,i)==0 && W_est(j,i)~=0
                        fp(m,i)=fp(m,i)+1;
                    end
                end
                %end
                p(i)=nnz(W2(:,i));  %calculate TPR and FPR for each task separately
                n(i)=dimt-p(i);
                tpr(m,i)=tp(m,i)/p(i);
                fpr(m,i)=fp(m,i)/n(i);
            end
            m=m+1;
        end
        %% LASSO
        for i=1:task_num
            opts=[];
            R1 = eye (task_num) - ones (task_num) / task_num;
            [W_est1, funcVal] = Least_SRMTL1(X_std{i}, Y_std{i}, R1, 0, p2, opts); %for R
            W(:,i)=W_est1;
        end
        % TPR and FPR for LASSO
        tp1(ls,:)=zeros(1,task_num);
        fp1(ls,:)=zeros(1,task_num);
        for i=1:task_num
            for j=1:dimt
                if W2(j,i)~=0 && W(j,i)~=0
                    tp1(ls,i)=tp1(ls,i)+1;
                end
                if W2(j,i)==0 && W(j,i)~=0
                    fp1(ls,i)=fp1(ls,i)+1;
                end
            end
            
            p1(i)=nnz(W2(:,i));  %calculate TPR and FPR for each task separately
            n1(i)=dimt-p1(i);
            tpr1(ls,i)=tp1(ls,i)/p1(i);
            fpr1(ls,i)=fp1(ls,i)/n1(i);
        end
        ls=ls+1;
    end
    %% draw ROC curve
    % delete the repeated measures
    mtcov=cell(task_num,1);
    Lsso=cell(task_num,1);
    for i=1:task_num
        r=[tpr(:,i) fpr(:,i)];
        mtcov{i}=unique(r,'rows');
        r1=[tpr1(:,i) fpr1(:,i)];
        Lsso{i}=unique(r1,'rows');
    end
    for i=1:task_num %
        % delete non-smooth points for mtcov
        %         ind=unique(mtcov{i}(:,1));
        %         l=length(ind);
        %         for j=1:l-1
        %             yid=find(mtcov{i}(:,1)==ind(l-j+1),1);
        %             cp=min(mtcov{i}(yid,2));
        %             yid1=find(mtcov{i}(:,1)==ind(l-j));
        %             g=length(yid1);
        %             for k=1:g
        %                 if mtcov{i}(yid1(g-k+1),2)>cp
        %                     mtcov{i}(yid1(g-k+1),:)=[];
        %                 end
        %             end
        %         end
        
        % delete non-smooth points for mtcov
        ind=unique(mtcov{i}(:,1));
        l=length(ind);
        minValue = 2;
        count = 1;
        for j=1:l
            yid=find(mtcov{i}(:,1)==ind(l-j+1));
            tempLength = length(yid);
            for k = 1:tempLength
                if(mtcov{i}(yid(k),2)<= minValue)
                    allValue(count, 1) = mtcov{i}(yid(k), 1);
                    allValue(count, 2) = mtcov{i}(yid(k),2);
                    count = count+1;
                end
            end
            if (minValue > min(mtcov{i}(yid,2)))
                minValue = min(mtcov{i}(yid,2));
            end
        end
        allValue = sort(allValue,1); % the smoothed points saved in allValue
        mtcov{i}=[];
        mtcov{i}=allValue;
        mtcov{i}=[[0 0];mtcov{i}]; % add (0,0) as the initial point for ROC
        
        
        % delete non-smooth points for Lsso
        ind1=unique(Lsso{i}(:,1));
        l1=length(ind1);
        for j=1:l1-1
            yid2=find(Lsso{i}(:,1)==ind1(l1-j+1),1);
            cp=Lsso{i}(yid2,2);
            yid3=find(Lsso{i}(:,1)==ind1(l1-j));
            g=length(yid3);
            for k=1:g
                if Lsso{i}(yid3(g-k+1),2)>cp
                    Lsso{i}(yid3(g-k+1),:)=[];
                end
            end
        end
        Lsso{i}=[[0 0];Lsso{i}]; % add (0,0) as the initial point for ROC
        Lsso{i}=[Lsso{i};[1 1]]; % add (1,1) as the end point for ROC
    end
    
    %% Area under Curve
    for i=1:task_num
        aoc=[];
        aoc=mtcov{i};
        aoc1=[];
        aoc1=Lsso{i};
        
        np=size(aoc,1);
        np1=size(aoc1,1);
        auc_mtcov(rp,i)=sum((aoc(2:np,2)-aoc(1:np-1,2)).*(aoc(2:np,1)+aoc(1:np-1,1)))/2; %area under roc
        auc_Lsso(rp,i)=sum((aoc1(2:np1,2)-aoc1(1:np1-1,2)).*(aoc1(2:np1,1)+aoc1(1:np1-1,1)))/2; %area under roc
    end
end
for i=1:task_num
    st_mtcov(:,i)=[mean(auc_mtcov(:,i));std(auc_mtcov(:,i))];
    st_Lsso(:,i)=[mean(auc_Lsso(:,i));std(auc_Lsso(:,i))];
end